package com.chelvc.framework.database.interceptor;

import java.sql.Connection;
import java.util.Map;

import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceAutoConfiguration;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.util.SQLUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.apache.ibatis.cache.CacheKey;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * 动态数据源MybatisPlus拦截器
 *
 * @author Woody
 * @date 2025/3/17
 */
@Component
@Intercepts({
        @Signature(type = StatementHandler.class, method = "getBoundSql", args = {}),
        @Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class}),
        @Signature(type = Executor.class, method = "update", args = {MappedStatement.class, Object.class}),
        @Signature(
                type = Executor.class, method = "query",
                args = {MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class}
        ),
        @Signature(
                type = Executor.class, method = "query",
                args = {
                        MappedStatement.class, Object.class, RowBounds.class,
                        ResultHandler.class, CacheKey.class, BoundSql.class
                }
        )
})
@ConditionalOnClass(DynamicDataSourceAutoConfiguration.class)
public class DynamicDataSourceMybatisPlusInterceptor extends CustomizeMybatisPlusInterceptor {
    private final String primary;
    private final Map<String, String> schemas;

    public DynamicDataSourceMybatisPlusInterceptor(@NonNull ApplicationContext applicationContext) {
        super(applicationContext);

        DynamicDataSourceProperties properties = applicationContext.getBean(DynamicDataSourceProperties.class);
        this.primary = properties.getPrimary();
        Map<String, DataSourceProperty> sources = properties.getDatasource();
        this.schemas = Maps.newHashMapWithExpectedSize(sources.size());
        sources.forEach((key, source) -> this.schemas.put(key, SQLUtils.getSchema(source.getUrl())));
    }

    @Override
    protected String getSchema() {
        return this.schemas.get(ObjectUtils.ifNull(DynamicDataSourceContextHolder.peek(), this.primary));
    }
}
